from django.db.backends.postgresql.schema import DatabaseSchemaEditor as DSE

class DatabaseSchemaEditor(DSE):

    sql_on_delete_cascade = " ON DELETE CASCADE "
    sql_on_delete_set_null = " ON DELETE SET NULL "

    sql_create_fk = (
        "ALTER TABLE %(table)s ADD CONSTRAINT %(name)s FOREIGN KEY (%(column)s) "
        "REFERENCES %(to_table)s (%(to_column)s)%(on_delete)s%(deferrable)s"
    )

    def _create_on_delete_sql(self, model, field, suffix):

        on_delete_db_cascade = getattr(field, "on_delete_db_cascade", False)
        on_delete_db_set_null = getattr(field, "on_delete_db_set_null", False)
        if on_delete_db_cascade:
            return self.sql_on_delete_cascade
        elif on_delete_db_set_null:
            return self.sql_on_delete_set_null
        else:
            return ""

    def _create_fk_sql(self, model, field, suffix):
        from_table = model._meta.db_table
        from_column = field.column
        to_table = field.target_field.model._meta.db_table
        to_column = field.target_field.column
        suffix = suffix % {
            "to_table": to_table,
            "to_column": to_column,
        }

        return self.sql_create_fk % {
            "table": self.quote_name(from_table),
            "name": self.quote_name(self._create_index_name(from_table, [from_column], suffix=suffix)),
            "column": self.quote_name(from_column),
            "to_table": self.quote_name(to_table),
            "to_column": self.quote_name(to_column),
            "deferrable": self.connection.ops.deferrable_sql(),
            "on_delete": self._create_on_delete_sql(model, field, suffix)
        }





from django.db.models import (
    ForeignKey as FK, OneToOneField as OTO,
    DO_NOTHING
)

from django_db_cascade.deletions import DB_CASCADE, DB_SET_NULL

### in actual django source we could just edit the class ForeignObject

class ForeignKey(FK):
    def __init__(self, to, on_delete, **kwargs):
        self.on_delete_db_cascade = False
        self.on_delete_db_set_null = False

        if on_delete == DB_CASCADE:
            self.on_delete_db_cascade = True
            on_delete = DO_NOTHING
        elif on_delete == DB_SET_NULL:
            self.on_delete_db_set_null = True
            on_delete = DO_NOTHING

        super(ForeignKey, self).__init__(to, on_delete, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(ForeignKey, self).deconstruct()
        if self.on_delete_db_cascade:
            kwargs['on_delete'] = DB_CASCADE
        elif self.on_delete_db_set_null:
            kwargs['on_delete'] = DB_SET_NULL
        return name, path, args, kwargs

class OneToOneField(OTO):
    def __init__(self, to, on_delete, **kwargs):
        self.on_delete_db_cascade = False
        self.on_delete_db_set_null = False

        if on_delete == DB_CASCADE:
            self.on_delete_db_cascade = True
            on_delete = DO_NOTHING
        elif on_delete == DB_SET_NULL:
            self.on_delete_db_set_null = True
            on_delete = DO_NOTHING
        super(OneToOneField, self).__init__(to, on_delete, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(OneToOneField, self).deconstruct()
        if self.on_delete_db_cascade:
            kwargs['on_delete'] = DB_CASCADE
        elif self.on_delete_db_set_null:
            kwargs['on_delete'] = DB_SET_NULL
        return name, path, args, kwargs




from setuptools import setup, find_packages

setup(
    name='django-db-cascade-3',
    version='0.3.1',
    description='Fork of https://github.com/jkapelner/django-db-cascade-2.  Optionally use postgres db ON CASCADE DELETE/SET NULL on django foreign keys',
    url='http://github.com/fingul/django-db-cascade-3',
    author='Fingul',
    author_email='fingul@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'Django >= 3.0',
        'psycopg2 >= 2.5'
    ],
    classifiers=[
        'Framework :: Django',
        'Topic :: Database',
    ],
    zip_safe=False
)
